#include "moglclass.h"
#include "CatmullRomCurveAnimation.h"

namespace moglclass
{

CatmullRomCurveAnimation::CatmullRomCurveAnimation(std::vector<KeyFrame> frames_, bool loop_)
{
    frames = frames_;
    loop = loop_;
	reset();
}

void CatmullRomCurveAnimation::reset()
{
	//elapsedTime = 0.0f;
	endTime = frames[frames.size() - 1].time;
    position = frames[0].position;
}

void CatmullRomCurveAnimation::setKeyframes(std::vector<KeyFrame> frames_)
{
	frames = frames_;
}

void CatmullRomCurveAnimation::updatePosition(float elapsedTime_)
{
	float elapsedTime = elapsedTime_;
	if (elapsedTime >= endTime)	// Loop around the total time if necessary
	{
		if (loop)
		{
			while (elapsedTime > endTime)
				elapsedTime -= endTime;
		}
		else
		{
			position = frames[frames.size() - 1].position;
			return;
		}
	}

	int i = 0;
	// Find the index of the current frame
	while (frames[i+1].time < elapsedTime)
		i++;
	
	// Find the time since the beginning of this frame
	elapsedTime -= frames[i].time;

	// Find how far we are between the current and next frame (0 to 1)
	float fraction = (float)(elapsedTime / (frames[i + 1].time - frames[i].time));
	//printf("fraction=%f \n ", fraction);

	// Interpolate position and rotation values between frames
	position = glm::catmullRom (frames[wrap(i - 1, frames.size() - 1)].position,
		frames[wrap(i, frames.size() - 1)].position,
		frames[wrap(i + 1, frames.size() - 1)].position,
		frames[wrap(i + 2, frames.size() - 1)].position, fraction);
	//printf("curve pos=%f %f %f\n ", position[0], position[1], position[2]);
}

// Wraps the "value" argument around [0, max]
int CatmullRomCurveAnimation::wrap(int value, int max)
{
    while (value > max) value -= max;
    while (value < 0) value += max;
    return value;
}

} // namespace